# Analyze Stock-Recruitment Data by fitting Hockey sticks with 3 different methods
# Developed by Rainer Froese and Gianpaolo Coro in September-November 2013

rm(list=ls(all=TRUE)) # clear previous variables etc
options(digits=3) # displays all numbers with three significant digits as default
library(R2jags)  # Interface with JAGS
library(FLCore)  # Fisheries Core Library
# get FLCore with: source("http://flr-project.org/R/instFLR.R")

# Conventions in naming parameters and variables
# RB_ is used as a prefix for rule-based parameters
# FL_ is used as a prefix for Fisheries Library parameters
# Ba_ is used as a prefix for Bayesian inference parameters 
# pr_ is used as a prefix for priors
# n. min. max. mean. SD. SE. LCL. UCL. tau. log. are used as prefix for the respective statistics

# get data
# The data file is expected as comma delimited text file with column headers:
# FishStock, Year, SSB, Recruitment,  RecruitmentAge
DataFile <- "S-R_AllStocks.csv" 
data.raw <- read.csv(DataFile, sep=",",header=T)
JAGSFILE="r2ssb.bug"
set.seed(123)

# write data to a csv file, define filename below, data are appended
OutFile <- "S-R_output.csv"

# select and rename columns to be used
data.all    <- data.frame(Stock=as.character(data.raw$FishStock), Year=as.numeric(data.raw$Year),
            SSB=as.numeric(data.raw$SSB), Rec=as.numeric(data.raw$Recruitment),
            RecAge=as.numeric(data.raw$RecruitmentAge),  
            stringsAsFactors=F)
# get alphabetic list of stocks
stocks <- sort(unique(data.all$Stock))
# stocks <- "whb-comb"  # "mac-nea"  use to examine a single stock

# loop through all stocks
for(stck in stocks) {
# select stock
data    <- data.all[data.all$Stock==stck,] 

cat("------------------------------------------------------------------------------------------- \n")
cat("Find best estimates for SSBlim and SSBpa from regular, rule-based and Bayesian hockey stick \n")
cat("------------------------------------------------------------------------------------------- \n")
cat("Data file =", DataFile, ", numbers displayed as thousands \n")
cat("Stock =", data$Stock[1], "\n")

# get age of recruits
t.rec   <- data$RecAge[1]

# select matching recruits (R) and parents (SSB)
R       <- data$Rec[(t.rec+1):length(data$SSB)]/1000 # Recruits in millions
SSB     <- data$SSB[1:(length(data$SSB)-t.rec)]/1000 # SSB in thousand tonnes

# establish some properties of the data
J           <- length(R) # number of observations
log.SSB      <- log(SSB)
max.SSB      <- max(SSB, na.rm=T)
min.SSB      <- min(SSB, na.rm=T)
log.R        <- log(R)

#------------------------------------------
# Estimate rule-based hockey stick
#------------------------------------------
# determine rule-based Rinf as geometric mean of recruitment at large stock sizes
RB_Rinf        <- exp(mean(log.R[which(SSB > (max.SSB+min.SSB)/2)], na.rm=T))
# determine rule-based SSBlim as biomass below which R < RB_Rinf
RB_SSBlim      <- min(SSB[which(R >= RB_Rinf)])

# max sure SSBlim is not smaller than 20% of max SSB, else use 20% max.SSB = SSBlim
RB_SSBlim      <- ifelse(RB_SSBlim < 0.2*max.SSB, 0.2*max.SSB, RB_SSBlim)  

# get stats on RB_Rinf               
n.RB_Rinf      <- length(log.R[which(SSB > (max.SSB+min.SSB)/2)])
SD.log.RB_Rinf  <- sd(log.R[which(SSB > (max.SSB+min.SSB)/2)], na.rm=T)
SE.log.RB_Rinf  <- SD.log.RB_Rinf/sqrt(n.RB_Rinf)
LCL.RB_Rinf    <- exp(log(RB_Rinf)-1.96*SE.log.RB_Rinf)
UCL.RB_Rinf    <- exp(log(RB_Rinf)+1.96*SE.log.RB_Rinf)
# get UCL for RB_SSBlim
# check whether there are R > UCL.RB_Rinf and RB_SSBlim > 20% of max biomass
if(length(SSB[which(R >= UCL.RB_Rinf)]) > 0 & RB_SSBlim > 0.2*max.SSB) {
  UCL.RB_SSBlim <- min(SSB[which(R >= UCL.RB_Rinf)])
} else { UCL.RB_SSBlim <- NA }

# determine RB_SSBpa from the upper confidence limit of RB_Rinf or as 1.4 * RB_SSBlim
RB_SSBpa      <- ifelse(is.na(UCL.RB_SSBlim)==T || 1.4*RB_SSBlim > UCL.RB_SSBlim, 1.4*RB_SSBlim, UCL.RB_SSBlim)
RB_SSBpa.type <- ifelse(RB_SSBlim == 0.2*max.SSB, "0.28 * max SSB", 
                     ifelse(RB_SSBpa==1.4*RB_SSBlim,"1.4 * RB_SSBlim", "upper CL of RB_Rinf") )
# determine RB_slope
RB_slope      <- RB_Rinf / RB_SSBlim

#-------------------------------------------
# Use FLR to fit regular hockey stick
#-------------------------------------------
library(FLCore)
# get the data into the FL object for stock-recruitment analysis, FLSR
# specify segmented regression = hockey stick as model
cat("Doing FLCore hockey-stick fitting, surpressing screen output...\n")
sr <- FLSR(rec=FLQuant(R, dimnames=list(year=data$Year[(t.rec+1):length(data$SSB)])),
           ssb=FLQuant(SSB, dimnames=list(year=data$Year[1:(length(data$SSB)-t.rec)])),
           model='segreg')

# use the fitting function fmle on the object sr
log   <- capture.output({  # supress output of fmle
    FL_sr <- fmle(sr);
})
# get the results from FLR with params() and vcov()
FL_slope     <- as.numeric(params(FL_sr)[1])  
sd.FL_slope  <- sqrt(as.numeric(vcov(FL_sr)[1,1,1])) # True???
FL_SSBlim    <- as.numeric(params(FL_sr)[2])
FL_Rinf      <- FL_slope * FL_SSBlim
# get upper 95% CI of FL_SSBlim from maximum likelihood theory
ll           <- logLik(FL_sr) # get log likelihood for the two parameters
log   <- capture.output({  # supress output of fmle
  fn           <- function(x,y) {(fmle(y, fixed=c(b=x))@logLik-(ll-qchisq(.95,1)/2))^2}; # function to minimize for 95% CL
  FL_SSBpa  <- optimise(f=fn, interval=c(params(FL_sr)["b",1,drop=T], params(FL_sr)["b",1,drop=T]*2.0),y=FL_sr)$minimum;
})
#----------------------------------------------------
# Bayesian Analysis
#----------------------------------------------------
cat("Doing Bayesian analysis with MCMC...\n")
# determine priors
pr_log.Ba_SSBlim      <- log(RB_SSBlim)
pr_SD.log.Ba_SSBlim   <- 0.072 * log(RB_SSBlim) # based on analysis of RB_SSBpa for 50 stocks
pr_tau.log.Ba_SSBlim  <- (pr_SD.log.Ba_SSBlim)^-2
pr_log.Ba_Rinf        <- log(RB_Rinf)
pr_SD.log.Ba_Rinf     <- 0.15*pr_log.Ba_Rinf # based on CV of RB_Rinf of 50 stocks 
pr_tau.log.Ba_Rinf    <- pr_SD.log.Ba_Rinf^-2 
pr_SD.SD.log.Ba_Rinf  <- 0.18*pr_log.Ba_Rinf # based on SD of CV of 50 stocks
pr_tau.SD.log.Ba_Rinf <- 1/pr_SD.SD.log.Ba_Rinf^2 

# simulation with Jags
Model = "
model {  
#priors
log.Rinf.ran     ~ dnorm(pr_log.Ba_Rinf, pr_tau.log.Ba_Rinf)
log.SSBlim.ran   ~ dnorm(pr_log.Ba_SSBlim,pr_tau.log.Ba_SSBlim)
SD.log.R         ~ dnorm(pr_SD.log.Ba_Rinf, pr_tau.SD.log.Ba_Rinf)
tau.log.R        <- pow(SD.log.R, -2)

for (j in 1:J){
logyh[j] <- ifelse(log.SSB[j] < log.SSBlim.ran, log.SSB[j] * log.Rinf.ran / log.SSBlim.ran, log.Rinf.ran)
log.R[j] ~ dnorm (logyh[j], tau.log.R) # data model:  the likelihood
  }
} 
"
# Write JAGS model 
cat(Model, file=JAGSFILE)  

#fit the model
jags.data <- list("J","pr_log.Ba_SSBlim","log.R","log.SSB", "pr_log.Ba_Rinf",
                  "pr_tau.log.Ba_Rinf","pr_tau.log.Ba_SSBlim","pr_SD.log.Ba_Rinf","pr_tau.SD.log.Ba_Rinf")
jags.params <- c("log.Rinf.ran","log.SSBlim.ran")

Nchains = 2
Nburnin = 1e3
Niter = 3e4
Nthin = 1

jagsfit <- jags(data=jags.data, working.directory=NULL, inits=NULL, jags.params, 
                model.file=JAGSFILE, n.chains=Nchains, n.thin=Nthin, n.iter=Niter, n.burnin=Nburnin)

#recover the outputs:
log.RinfBest<-jagsfit$BUGSoutput$sims.list$log.Rinf.ran
log.SSBlimBest<-jagsfit$BUGSoutput$sims.list$log.SSBlim.ran

#estimated SSBlim and Rinf according to MCMC
Ba_SSBlim         <- exp(mean(log.SSBlimBest))  # geometric mean of MCMC SSBlim estimates
sd.log.SSBlim      <- apply(as.matrix(log.SSBlimBest),2,sd) 
LCL.Ba_SSBlim     <- exp(log(Ba_SSBlim)-1.96*sd.log.SSBlim)
Ba_SSBpa          <- exp(log(Ba_SSBlim)+1.96*sd.log.SSBlim)
Ba_Rinf           <- exp(mean(log.RinfBest))
sd.log.Rinf        <- apply(as.matrix(log.RinfBest),2,sd) 
LCL.Ba_Rinf       <- exp(log(Ba_Rinf)-1.96*sd.log.Rinf)
UCL.Ba_Rinf       <- exp(log(Ba_Rinf)+1.96*sd.log.Rinf)
Ba_slope          <- Ba_Rinf / Ba_SSBlim

cat("Results \n")
cat("--------\n")
cat("min SSB =", min(SSB, na.rm=T), ", max SSB =", max(SSB, na.rm=T), "\n")
cat("FLR-based hockey stick \n")
cat("FL_SSBlim =", FL_SSBlim, ", upper 95% CL = FL_SSBpa =", FL_SSBpa,"\n")
cat("FL_slope =", FL_slope, "\n")
cat("FL_Rinf =", FL_Rinf, "\n")
cat("Rule-based hockey stick \n")
cat("RB_SSBlim =", RB_SSBlim, "\n")
cat("RB_SSBpa = ", RB_SSBpa,  "derived from", RB_SSBpa.type, "\n")
cat("RB_slope =", RB_slope, "\n")
cat("RB_Rinf =", RB_Rinf,"(", LCL.RB_Rinf, "-", UCL.RB_Rinf, ")\n")
cat("Bayesian hockey stick \n")
cat("Ba_SSBlim    :",Ba_SSBlim, "(", LCL.Ba_SSBlim, "-", Ba_SSBpa,")\n")
cat("Ba_SSBpa     :",Ba_SSBpa,"\n")
cat("Ba_slope     :",Ba_slope, "\n")
cat("Ba_Rinf      :",Ba_Rinf,"(", LCL.Ba_Rinf, "-", UCL.Ba_Rinf, ")\n")

# quality check
max.SSBlim <- max(c(FL_SSBlim,RB_SSBlim,Ba_SSBlim))
if(min(SSB) >= max.SSBlim) {
  cat("Warning: biomass data >= max SSBlim \n")
}
min.SSBpa <- min(c(FL_SSBpa,RB_SSBpa,Ba_SSBpa))
if(max(SSB) <= min.SSBpa) {
  cat("Warning: biomass data <= min SSBpa \n")
}


# Plots:
# plot rule-based HS
#for desktop usage only
max.SSBpa <- max(c(FL_SSBpa,RB_SSBpa,Ba_SSBpa))
upper.xlim <- ifelse(max.SSBpa > max(SSB),max.SSBpa,max(SSB))
plot(x=SSB, y=R, bty="l", xlim=c(0, upper.xlim),  ylim=c(0, max(R, na.rm=T)), 
     xlab="SSB", ylab="Recruits", main=paste("Rule-based (black), Bayesian (red), 
     and FLR (green) hockey sticks for", stck))
lines(x=c(0,RB_SSBlim,upper.xlim), y=c(0.0001,RB_Rinf,RB_Rinf))
abline(v=RB_SSBpa, lty=3, lwd=1.5)

#plot the Bayesian HS
abline(v=Ba_SSBpa, lty=3, col="red",lwd=1.5)
lines(x=c(0,Ba_SSBlim,upper.xlim), y=c(0.0001,Ba_Rinf,Ba_Rinf), lwd=1.5, col="red")

# plot Fisheries Library HS
lines(x=c(0, FL_SSBlim, upper.xlim), y=c(0.0001, FL_Rinf, FL_Rinf), lwd=1.5, col="green")
abline(v=FL_SSBpa, lty=3, lwd=1.5, col="green")


# write results
out <- data.frame(data$Stock[1], J, min.SSB, max.SSB, FL_SSBlim, 
        FL_SSBpa, FL_slope, RB_SSBlim, RB_SSBpa, RB_slope, RB_Rinf, SD.log.RB_Rinf, Ba_SSBlim, 
        Ba_SSBpa, Ba_Rinf, Ba_slope)
write.table(out, OutFile, append=T, row.names=F, col.names=F, sep=",", dec=".")

} # end of loop for all stocks



